#!/usr/local/bin/ruby
# sings.rb - simple SignServer 2
#
# $Date: 2000/09/11 17:00:19 $
# Copyright (c) 2000 Masatoshi SEKI
#
# pserv.rb is copyrighted free software by Masatoshi SEKI.
# You can redistribute it and/or modify it under the same term as Ruby.

require 'drb/drb'
require 'drbauth'
require 'mutexm'

class SignDB
  include MutexM
  def initialize(fname)
    super()
    @fname = fname
    @dict = {}
    begin
      fp = File.open(@fname, 'r')
      @dict = Marshal.load(fp)
    rescue
      @dict = {}
    ensure
      fp.close if fp
    end
    self['adm'] = 'adm' if @dict.empty?
  end

  def [](u)
    synchronize do 
      return @dict[u]
    end
  end
  
  def []=(u, pass)
    synchronize do 
      @dict[u] = DRbAuth.digest(u, pass)
      File.open(@fname, 'w') do |fp|
	Marshal.dump(@dict, fp)
      end
    end
  end
end

class SignAdmin
  include DRbUndumped
  
  POOL = []			# for GC

  def SignAdmin.push(it)
    POOL.push(it)
  end

  def SignAdmin.delete(it)
    POOL.delete(it)
  end

  def initialize(db, user)
    @db = db
    @user = user
    SignAdmin.push(self)
  end
  
  def admin?
    @user == 'adm'
  end

  def set_pass(user, pass)
    raise DRbAuth::DRbAuthError if @user != user && !admin?
    @db[user] = pass
    @user = nil
    SignAdmin.delete(self)
  end

  def add_user(user, pass)
    raise DRbAuth::DRbAuthError unless admin?
    @db[user] = pass
    @user = nil
    SignAdmin.delete(self)
  end
end

class SignServer
  def initialize(dict)
    @dict = dict
  end

  def check_sign(ticket)
    salt = Time.now.to_s
    name, g_sign, opt = ticket.sign(salt)
    if @dict[name].nil? or DRbAuth::digest(@dict[name], salt) != g_sign
      raise DRbAuth::DRbAuthError 
    end
    [name, opt]
  end

  def checkout(ticket)
    name, opt = check_sign(ticket)
    ticket.store(SignAdmin.new(@dict, name))
  end
end

if __FILE__ == $0
  db = SignDB.new('stest.dat')
  ts = SignServer.new(db)

  DRb.start_service('druby://:8471', ts)
  puts DRb.uri
  if $DEBUG
    gets
    exit
  else
    DRb.thread.join
  end
end
